Création d'un serveur de service avec Falsk et Python
=====================================================
![category](développement)
![tag](python)
![tag](flask)

Cette documentation présente le résultat d'une recherche  de solution autours
de la mise en place d'un serveur de service


Objectif
--------

Le serveur de service doit avoir pour fonctionnalité

- intégration automatique de nouveau service
- retour en arrière simple après la mise en Service
- generation de documentation

La gestion de production
------------------------

Très rapidement, il a été identifié comme solution:

- utilisation du couple Apache/mod_wsgi
- utilisation du framework Flask (avec extension flask-autodoc)
- mise en place d'un serveur de déploiement pip interne (pypiserver)
- installation des services via **pip** (un ensemble de service étant contenu dans un module)

L'objectif est que notre framework web puisse charger de façon dynamique l'ensemble
des modules nommé MyCompany??? contenant un sous-module route (contenant les services
à déployer)

Le processus est donc le suivant:

- je développe mon module contenant mes services
- je déploie mon module sur le serveur de déploiement
- j'installe mon module sur le serveur de production contenant mes web-service
- je redémarre Apache (ou je gère un service qui recharge les services)

Résultat
--------

Le module framework
~~~~~~~~~~~~~~~~~~~

le module se nomme MyCompanyFrameworkServ

mycompanyframeworkserv/__init__.py

```python
__version_info__ = (0, 0, 1)
__version__ = '.'.join(map(str, __version_info__))
```

mycompanyframeworkserv/conf.py

```python
import os
import logging

DEBUG = True
SECRET_KEY = 'secret_key'
PORT = 5000
HOST = '0.0.0.0'
LEVEL_LOG = logging.DEBUG

SPACENAME_SERVICE = 'mycompanysrv'

```

mycompanyframeworkserv/server.py

```python
import os, os.path
import logging

from flask import Flask
from flask.ext.autodoc import Autodoc
import pip

# Flask configuration

app = Flask(__name__)
app.config.from_pyfile('conf.py')
app.doc = Autodoc(app)

handler = logging.StreamHandler()
handler.setLevel(app.config['LEVEL_LOG'] or logging.DEBUG)
app.logger.addHandler(handler)

# load dynamic of package
installed_packages = pip.get_installed_distributions()
for mod in installed_packages:
    if mod.key[:len(app.config['SPACENAME_SERVICE'])] == app.config['SPACENAME_SERVICE']:
        app.logger.info('import %s' % mod.key.lower())
        pkg = __import__('sodebosrvart', globals(), locals(), ['route'])
        app.logger.debug('load route of %s' % mod.key.lower())
        pkg.route.load(app)

@app.route('/documentation')
def documentation():
    return app.doc.html()

if __name__ == "__main__":
    app.run(host = app.config['HOST'],
        port = app.config['PORT'])

```

Le module Service
~~~~~~~~~~~~~~~~~

le module se nomme MyCompanySrvArt

mycompanysrvart/__init__.py

```python
__version_info__ = (0, 0, 1)
__version__ = '.'.join(map(str, __version_info__))
```

mycompanysrvart/route.py

```python
#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import osa
import sodebosrvart
import sodebosrvart.service as service
import flask
import json

def load(app):

    # a vous de rajouter des routes ;-)

    @app.route("/art/version", methods = ['GET','POST'])
    @app.doc.doc()
    def getversion():
        " retour la version du module sodebosrvart au format json"
        return  json.dumps({"version": sodebosrvart.__version__})```
